@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import itertools
 
@@ -95,6 +90,15 @@ def test_can_mark_invalid():
     assert x.status == Status.INVALID
 
 
+def test_can_mark_invalid_with_why():
+    x = ConjectureData.for_buffer(b"")
+    with pytest.raises(StopTest):
+        x.mark_invalid("some reason")
+    assert x.frozen
+    assert x.status == Status.INVALID
+    assert x.events == {"some reason"}
+
+
 class BoomStrategy(SearchStrategy):
     def do_draw(self, data):
         data.draw_bytes(1)
@@ -111,7 +115,7 @@ def test_closes_interval_on_error_in_strategy():
 
 class BigStrategy(SearchStrategy):
     def do_draw(self, data):
-        data.draw_bytes(10 ** 6)
+        data.draw_bytes(10**6)
 
 
 def test_does_not_double_freeze_in_interval_close():
@@ -338,7 +342,7 @@ def test_will_mark_too_deep_examples_as_invalid():
 
     s = st.none()
     for _ in range(MAX_DEPTH + 1):
-        s = s.map(lambda x: x)
+        s = s.map(lambda x: None)
 
     with pytest.raises(StopTest):
         d.draw(s)
