@@ -671,7 +671,12 @@ def test_resolving_recursive_type_with_registered_constraint():
     with temp_registered(
         SomeClass, st.builds(SomeClass, value=st.integers(min_value=1))
     ):
-        find_any(st.from_type(SomeClass), lambda s: s.next_node is None)
+
+        @given(s=st.from_type(SomeClass))
+        def test(s):
+            assert isinstance(s, SomeClass)
+
+        test()
 
 
 def test_resolving_recursive_type_with_registered_constraint_not_none():
