@@ -1,17 +1,12 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import datetime
 import subprocess
@@ -22,6 +17,7 @@ import pytest
 
 from hypothesis import example, given, strategies as st
 from hypothesis._settings import (
+    HealthCheck,
     Phase,
     Verbosity,
     default_variable,
@@ -30,10 +26,20 @@ from hypothesis._settings import (
     settings,
 )
 from hypothesis.database import ExampleDatabase
-from hypothesis.errors import InvalidArgument, InvalidState
+from hypothesis.errors import (
+    HypothesisDeprecationWarning,
+    InvalidArgument,
+    InvalidState,
+)
 from hypothesis.stateful import RuleBasedStateMachine, rule
 from hypothesis.utils.conventions import not_set
-from tests.common.utils import counts_calls, fails_with
+
+from tests.common.utils import (
+    checks_deprecated_behaviour,
+    counts_calls,
+    fails_with,
+    validate_deprecation,
+)
 
 
 def test_has_docstrings():
@@ -187,11 +193,12 @@ def test_can_have_none_database():
 
 
 @pytest.mark.parametrize("db", [None, ExampleDatabase(":memory:")])
-def test_database_type_must_be_ExampleDatabase(db):
+@pytest.mark.parametrize("bad_db", [":memory:", ".hypothesis/examples"])
+def test_database_type_must_be_ExampleDatabase(db, bad_db):
     with local_settings(settings(database=db)):
         settings_property_db = settings.database
         with pytest.raises(InvalidArgument):
-            settings(database=".hypothesis/examples")
+            settings(database=bad_db)
         assert settings.database is settings_property_db
 
 
@@ -276,10 +283,9 @@ if __name__ == '__main__':
 """
 
 
-def test_puts_the_database_in_the_home_dir_by_default(tmpdir):
-    script = tmpdir.join("assertlocation.py")
-    script.write(ASSERT_DATABASE_PATH)
-
+def test_puts_the_database_in_the_home_dir_by_default(tmp_path):
+    script = tmp_path.joinpath("assertlocation.py")
+    script.write_text(ASSERT_DATABASE_PATH, encoding="utf-8")
     subprocess.check_call([sys.executable, str(script)])
 
 
@@ -417,7 +423,7 @@ def test_assigning_to_settings_attribute_on_state_machine_raises_error():
 
         class StateMachine(RuleBasedStateMachine):
             @rule(x=st.none())
-            def a_rule(x):
+            def a_rule(self, x):
                 assert x is None
 
         StateMachine.settings = settings()
@@ -463,13 +469,37 @@ def test_invalid_parent():
 
 
 def test_show_changed():
-    s = settings(max_examples=999, database=None)
+    s = settings(max_examples=999, database=None, phases=tuple(Phase)[:-1])
     assert s.show_changed() == "database=None, max_examples=999"
 
 
 def test_note_deprecation_checks_date():
-    with pytest.warns(None) as rec:
-        note_deprecation("This is bad", since="RELEASEDAY")
+    with pytest.warns(HypothesisDeprecationWarning) as rec:
+        note_deprecation("This is bad", since="RELEASEDAY", has_codemod=False)
     assert len(rec) == 1
     with pytest.raises(AssertionError):
-        note_deprecation("This is way too old", since="1999-12-31")
+        note_deprecation("This is way too old", since="1999-12-31", has_codemod=False)
+
+
+def test_note_deprecation_checks_has_codemod():
+    with pytest.warns(
+        HypothesisDeprecationWarning,
+        match="The `hypothesis codemod` command-line tool",
+    ):
+        note_deprecation("This is bad", since="2021-01-01", has_codemod=True)
+
+
+def test_deprecated_settings_warn_on_set_settings():
+    with validate_deprecation():
+        settings(suppress_health_check=[HealthCheck.return_value])
+    with validate_deprecation():
+        settings(suppress_health_check=[HealthCheck.not_a_test_method])
+
+
+@checks_deprecated_behaviour
+def test_deprecated_settings_not_in_settings_all_list():
+    al = HealthCheck.all()
+    ls = list(HealthCheck)
+    assert al == ls
+    assert HealthCheck.return_value not in ls
+    assert HealthCheck.not_a_test_method not in ls
