@@ -1,24 +1,21 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import pytest
 
-from hypothesis import Phase, assume, given, settings, strategies as st
+from hypothesis import Phase, assume, given, settings, strategies as st, target
 from hypothesis.database import InMemoryExampleDatabase
-from hypothesis.errors import Flaky, MultipleFailures
+from hypothesis.errors import Flaky
+from hypothesis.internal.compat import ExceptionGroup
 from hypothesis.internal.conjecture.engine import MIN_TEST_CALLS
+
 from tests.common.utils import (
     assert_output_contains_failure,
     capture_out,
@@ -26,6 +23,16 @@ from tests.common.utils import (
 )
 
 
+def capture_reports(test):
+    with capture_out() as o, pytest.raises(ExceptionGroup) as err:
+        test()
+
+    return o.getvalue() + "\n\n".join(
+        f"{e!r}\n" + "\n".join(getattr(e, "__notes__", []))
+        for e in (err.value,) + err.value.exceptions
+    )
+
+
 def test_raises_multiple_failures_with_varying_type():
     target = [None]
 
@@ -42,12 +49,20 @@ def test_raises_multiple_failures_with_varying_type():
         exc_class = TypeError if target[0] == i else ValueError
         raise exc_class()
 
-    with capture_out() as o:
-        with pytest.raises(MultipleFailures):
-            test()
+    output = capture_reports(test)
+    assert "TypeError" in output
+    assert "ValueError" in output
 
-    assert "TypeError" in o.getvalue()
-    assert "ValueError" in o.getvalue()
+
+def test_shows_target_scores_with_multiple_failures():
+    @settings(derandomize=True, max_examples=10_000)
+    @given(st.integers())
+    def test(i):
+        target(i)
+        assert i > 0
+        assert i < 0
+
+    assert "Highest target score:" in capture_reports(test)
 
 
 def test_raises_multiple_failures_when_position_varies():
@@ -65,11 +80,9 @@ def test_raises_multiple_failures_when_position_varies():
         else:
             raise ValueError("loc 2")
 
-    with capture_out() as o:
-        with pytest.raises(MultipleFailures):
-            test()
-    assert "loc 1" in o.getvalue()
-    assert "loc 2" in o.getvalue()
+    output = capture_reports(test)
+    assert "loc 1" in output
+    assert "loc 2" in output
 
 
 def test_replays_both_failing_values():
@@ -85,10 +98,10 @@ def test_replays_both_failing_values():
         exc_class = TypeError if target[0] == i else ValueError
         raise exc_class()
 
-    with pytest.raises(MultipleFailures):
+    with pytest.raises(ExceptionGroup):
         test()
 
-    with pytest.raises(MultipleFailures):
+    with pytest.raises(ExceptionGroup):
         test()
 
 
@@ -115,7 +128,7 @@ def test_replays_slipped_examples_once_initial_bug_is_fixed(fix):
         if i == target[1]:
             raise ValueError()
 
-    with pytest.raises(MultipleFailures):
+    with pytest.raises(ExceptionGroup):
         test()
 
     bug_fixed = True
@@ -146,7 +159,7 @@ def test_garbage_collects_the_secondary_key():
         if i == target[1]:
             raise ValueError()
 
-    with pytest.raises(MultipleFailures):
+    with pytest.raises(ExceptionGroup):
         test()
 
     bug_fixed = True
@@ -172,7 +185,7 @@ def test_shrinks_both_failures():
     def test(i):
         if i >= 10000:
             first_has_failed[0] = True
-            assert False
+            raise AssertionError
         assert i < 10000
         if first_has_failed[0]:
             if second_target[0] is None:
@@ -184,11 +197,7 @@ def test_shrinks_both_failures():
         else:
             duds.add(i)
 
-    with capture_out() as o:
-        with pytest.raises(MultipleFailures):
-            test()
-
-    output = o.getvalue()
+    output = capture_reports(test)
     assert_output_contains_failure(output, test, i=10000)
     assert_output_contains_failure(output, test, i=second_target[0])
 
@@ -216,13 +225,19 @@ def test_handles_flaky_tests_where_only_one_is_flaky():
             flaky_failed_once[0] = True
             raise ValueError()
 
-    with pytest.raises(Flaky):
+    try:
         test()
+        raise AssertionError("Expected test() to raise an error")
+    except ExceptionGroup as err:
+        assert any(isinstance(e, Flaky) for e in err.exceptions)
 
     flaky_fixed = True
 
-    with pytest.raises(MultipleFailures):
+    try:
         test()
+        raise AssertionError("Expected test() to raise an error")
+    except ExceptionGroup as err:
+        assert not any(isinstance(e, Flaky) for e in err.exceptions)
 
 
 @pytest.mark.parametrize("allow_multi", [True, False])
@@ -241,7 +256,7 @@ def test_can_disable_multiple_error_reporting(allow_multi):
             seen.add(ValueError)
             raise ValueError
 
-    with pytest.raises(MultipleFailures if allow_multi else TypeError):
+    with pytest.raises(ExceptionGroup if allow_multi else TypeError):
         test()
     assert seen == {TypeError, ValueError}
 
@@ -250,7 +265,7 @@ def test_finds_multiple_failures_in_generation():
     special = []
     seen = set()
 
-    @settings(phases=[Phase.generate], max_examples=100)
+    @settings(phases=[Phase.generate, Phase.shrink], max_examples=100)
     @given(st.integers(min_value=0))
     def test(x):
         """Constructs a test so the 10th largeish example we've seen is a
@@ -267,7 +282,7 @@ def test_finds_multiple_failures_in_generation():
             assert x in seen or (x <= special[0])
         assert x not in special
 
-    with pytest.raises(MultipleFailures):
+    with pytest.raises(ExceptionGroup):
         test()
 
 
@@ -278,7 +293,7 @@ def test_stops_immediately_if_not_report_multiple_bugs():
     @given(st.integers())
     def test(x):
         seen.add(x)
-        assert False
+        raise AssertionError
 
     with pytest.raises(AssertionError):
         test()
@@ -288,7 +303,7 @@ def test_stops_immediately_if_not_report_multiple_bugs():
 def test_stops_immediately_on_replay():
     seen = set()
 
-    @settings(database=InMemoryExampleDatabase())
+    @settings(database=InMemoryExampleDatabase(), phases=tuple(Phase)[:-1])
     @given(st.integers())
     def test(x):
         seen.add(x)
