@@ -1,21 +1,17 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import os
 import shlex
 import subprocess
+from pathlib import Path
 
 
 def current_branch():
@@ -34,18 +30,14 @@ def tags():
     return set(result)
 
 
-ROOT = (
+ROOT = Path(
     subprocess.check_output(
         ["git", "-C", os.path.dirname(__file__), "rev-parse", "--show-toplevel"]
     )
     .decode("ascii")
     .strip()
 )
-
-
-REPO_TESTS = os.path.join(ROOT, "whole-repo-tests")
-
-PYUP_FILE = os.path.join(ROOT, ".pyup.yml")
+REPO_TESTS = ROOT / "whole-repo-tests"
 
 
 def hash_for_name(name):
@@ -97,14 +89,12 @@ def git(*args):
     subprocess.check_call(("git",) + args)
 
 
-TOOLING_COMMITER_NAME = "Travis CI on behalf of David R. MacIver"
+TOOLING_COMMITER_NAME = "CI on behalf of the Hypothesis team"
 
 
 def configure_git():
     git("config", "user.name", TOOLING_COMMITER_NAME)
     git("config", "user.email", "david@drmaciver.com")
-    git("config", "core.sshCommand", "ssh -i deploy_key")
-    git("remote", "add", "ssh-origin", "git@github.com:HypothesisWorks/hypothesis.git")
 
 
 def create_tag(tagname):
@@ -114,25 +104,12 @@ def create_tag(tagname):
 
 def push_tag(tagname):
     assert_can_release()
-    subprocess.check_call(
-        [
-            "ssh-agent",
-            "sh",
-            "-c",
-            "ssh-add %s && " % (shlex.quote(DEPLOY_KEY),)
-            + "git push ssh-origin HEAD:master &&"
-            + "git push ssh-origin %s" % (shlex.quote(tagname),),
-        ]
-    )
+    subprocess.check_call(["git", "push", "origin", shlex.quote(tagname)])
+    subprocess.check_call(["git", "push", "origin", "HEAD:master"])
 
 
 def assert_can_release():
     assert not IS_PULL_REQUEST, "Cannot release from pull requests"
-    assert has_travis_secrets(), "Cannot release without travis secure vars"
-
-
-def has_travis_secrets():
-    return os.environ.get("TRAVIS_SECURE_ENV_VARS", None) == "true"
 
 
 def modified_files():
@@ -151,8 +128,7 @@ def modified_files():
         diff_output = subprocess.check_output(command).decode("ascii")
         for l in diff_output.split("\n"):
             filepath = l.strip()
-            if filepath:
-                assert os.path.exists(filepath), filepath
+            if filepath and os.path.exists(filepath):
                 files.add(filepath)
     return files
 
@@ -180,58 +156,12 @@ def changed_files_from_master():
     return files
 
 
-SECRETS_BASE = os.path.join(ROOT, "secrets")
-SECRETS_TAR = SECRETS_BASE + ".tar"
-ENCRYPTED_SECRETS = SECRETS_TAR + ".enc"
-
-SECRETS = os.path.join(ROOT, "secrets")
-
-DEPLOY_KEY = os.path.join(SECRETS, "deploy_key")
-PYPIRC = os.path.join(SECRETS, ".pypirc")
-
-CARGO_API_KEY = os.path.join(SECRETS, "cargo-credentials")
-
-
-SECRET_FILES = [DEPLOY_KEY, PYPIRC, CARGO_API_KEY]
-
-
-def decrypt_secrets():
-    subprocess.check_call(
-        [
-            "openssl",
-            "aes-256-cbc",
-            "-K",
-            os.environ["encrypted_b8618e5d043b_key"],
-            "-iv",
-            os.environ["encrypted_b8618e5d043b_iv"],
-            "-in",
-            ENCRYPTED_SECRETS,
-            "-out",
-            SECRETS_TAR,
-            "-d",
-        ]
-    )
-
-    subprocess.check_call(["tar", "-xvf", SECRETS_TAR], cwd=ROOT)
-
-    missing_files = [os.path.basename(f) for f in SECRET_FILES if not os.path.exists(f)]
-
-    assert not missing_files, missing_files
-    os.chmod(DEPLOY_KEY, int("0600", 8))
-
-
-IS_TRAVIS_PULL_REQUEST = os.environ.get("TRAVIS_EVENT_TYPE") == "pull_request"
-
-IS_CIRCLE_PULL_REQUEST = (
-    os.environ.get("CIRCLE_BRANCH") == "master"
-    and os.environ.get("CI_PULL_REQUESTS", "") != ""
-)
-
-
-IS_PULL_REQUEST = IS_TRAVIS_PULL_REQUEST or IS_CIRCLE_PULL_REQUEST
+IS_PULL_REQUEST = os.environ.get("GITHUB_REF", "").startswith("refs/pull/")
 
 
 def all_projects():
-    from hypothesistooling.projects import conjecturerust as cr, hypothesispython as hp
+    import hypothesistooling.projects.conjecturerust as cr
+    import hypothesistooling.projects.hypothesispython as hp
+    import hypothesistooling.projects.hypothesisruby as hr
 
-    return [cr, hp]
+    return [cr, hp, hr]
